#pragma once

/** 96bit (3 float RGB) floating point color element.

	This is the native format WM uses for bitmap manipulation.	
*/
struct rgb96_data_element {
public:
	rgb96_data_element() { };
	rgb96_data_element(float _r, float _g, float _b) { r = _r; g = _g; b = _b; };
	void CreateFrom(DWORD rgb);	///< Import RGB value from a packed 32bit RGB value.
	DWORD GetPacked();			///< Export packed RGB value

	rgb96_data_element operator+( const rgb96_data_element &second );
	rgb96_data_element operator-( const rgb96_data_element &second );
	rgb96_data_element operator*( const rgb96_data_element vector );
	rgb96_data_element operator*( const float scalar );
	rgb96_data_element operator/( const rgb96_data_element vector );
	rgb96_data_element operator/( const float scalar );
	rgb96_data_element &operator+=( const rgb96_data_element &second);
	rgb96_data_element &operator*=( const float scalar );
	rgb96_data_element &operator/=( const float scalar );
	float r,g,b;
};

/// colorF is shorthand for the rather long rgb96_data_element
typedef rgb96_data_element colorF;

namespace WorldMachine {

/// Create a HSL triplet from an RGB triplet
colorF hsl(colorF c);

/// Create a RGB triplet from an HSL triplet
colorF rgb(colorF hsl);

};


// implementation friend funcs

inline rgb96_data_element operator*( const float scalar, const rgb96_data_element &value ) {
	rgb96_data_element result;
	result.r = value.r * scalar;
	result.g = value.g * scalar;
	result.b = value.b * scalar;
	return result;
};


inline rgb96_data_element rgb96_data_element::operator+( const rgb96_data_element &other ) {
	rgb96_data_element result;
	result.r = r + other.r;
	result.g = g + other.g;
	result.b = b + other.b;
	return result;
};

inline rgb96_data_element rgb96_data_element::operator-( const rgb96_data_element &other ) {
	rgb96_data_element result;
	result.r = r - other.r;
	result.g = g - other.g;
	result.b = b - other.b;
	return result;
};

inline rgb96_data_element rgb96_data_element::operator*( const float scalar ) {
	rgb96_data_element result;
	result.r = r * scalar;
	result.g = g * scalar;
	result.b = b * scalar;
	return result;
};

inline rgb96_data_element rgb96_data_element::operator*( const rgb96_data_element vect ) {
	rgb96_data_element result;
	result.r = r * vect.r;
	result.g = g * vect.g;
	result.b = b * vect.b;
	return result;
};

inline rgb96_data_element rgb96_data_element::operator/( const float scalar ) {
	rgb96_data_element result;
	result.r = r / scalar;
	result.g = g / scalar;
	result.b = b / scalar;
	return result;
};

inline rgb96_data_element rgb96_data_element::operator/( const rgb96_data_element vect ) {
	rgb96_data_element result;
	result.r = r / vect.r;
	result.g = g / vect.g;
	result.b = b / vect.b;
	return result;
};

inline rgb96_data_element &rgb96_data_element::operator+=( const rgb96_data_element &second) {
	r += second.r;
	g += second.g;
	b += second.b;
	return *this;
};

inline rgb96_data_element &rgb96_data_element::operator*=( const float s) {
	r *= s;
	g *= s;
	b *= s;
	return *this;
};

inline rgb96_data_element &rgb96_data_element::operator/=( const float s) {
	r /= s;
	g /= s;
	b /= s;
	return *this;
};

